# Минимальное Остовное Дерево и Гамильтонов Цикл

Этот проект реализует алгоритмы для нахождения минимального остовного дерева (MST) и Гамильтонова цикла в графах. Он включает в себя три основных класса: `Kruskal`, `Hamilton` и `Primal_min`, каждый из которых предоставляет различные методы для работы с графами.

## Установка

Для работы с проектом необходимо установить следующие библиотеки:

```bash
pip install networkx matplotlib
```

## Описание классов

### 1. Kruskal

Класс `Kruskal` реализует алгоритм Краскала для нахождения минимального остовного дерева.

#### Конструктор

```python
Kruskal(points, edges)
```

- `points`: Список координат вершин графа.
- `edges`: Список рёбер графа, где каждое ребро представлено в виде кортежа (u, v, weight).

#### Методы

- `draw_only(k)`: Отображает граф и его рёбра.
- `view(min_edges, max_edges, k)`: Отображает граф с минимальным и максимальным покрывающим деревом.
- `kruskals_algorithm(edges)`: Реализует алгоритм Краскала для нахождения MST.
- `sort_edges_min()`: Сортирует рёбра по возрастанию веса.
- `sort_edges_max()`: Сортирует рёбра по убыванию веса.
- `result_weight(edges)`: Вычисляет и выводит суммарный вес дерева.

### 2. Hamilton

Класс `Hamilton` реализует алгоритм для нахождения Гамильтонова цикла.

#### Конструктор

```python
Hamilton(points, edges)
```

- `points`: Список координат вершин графа.
- `edges`: Список рёбер графа в виде пар индексов вершин.

#### Методы

- `cycle_exist(cycle)`: Проверяет наличие Гамильтонова цикла и выводит его.
- `draw()`: Визуализирует граф.
- `aviable()`: Выводит доступные вершины.
- `hamiltonian_cycle(start)`: Находит Гамильтонов цикл, начиная с заданной вершины.
- `draw_graph(path=None)`: Отображает граф с выделенным Гамильтоновым циклом.

### 3. Primal_min

Класс `Primal_min` реализует алгоритм Прима для нахождения минимального остовного дерева.

#### Конструктор

```python
Primal_min(graph)
```

- `graph`: Граф в виде словаря, где ключи — это вершины, а значения — списки рёбер с весами.

#### Методы

- `run(begin)`: Запускает алгоритм Прима, начиная с заданной вершины.
- `min_tree()`: Выводит минимальное покрывающее дерево и его суммарный вес.

## Пример использования

```python
# Пример создания графа и нахождения минимального остовного дерева
points = [(0, 0), (1, 1), (2, 0), (1, -1)]
edges = [(0, 1, 1.5), (0, 2, 1.0), (1, 3, 2.0), (2, 3, 1.0)]

kruskal = Kruskal(points, edges)
sorted_edges = kruskal.sort_edges_min()
mst = kruskal.alg_Kraskala(sorted_edges)
kruskal.view(mst, sorted_edges, 3)

# Пример нахождения Гамильтонова цикла
hamilton = Hamilton(points, edges)
cycle = hamilton.hamiltonian_cycle(0)
hamilton.cycle_exist(cycle)
hamilton.draw_graph(cycle)
```

## Лицензия

Этот проект лицензирован под MIT License. Пожалуйста, ознакомьтесь с файлом LICENSE для получения дополнительной информации.
```